;;; emms-player-extensions.el - Add more user control functions for EMMS

;; Copyright (C) 2005 William XWL

;; Author: William XWL <william.xwl@gmail.com>
;; Version: v 0.21
;; Keywords: emms music mp3
;; Last updated: 2005/07/20 23:42:13

;; This file is not part of GNU Emacs.

;;{{{ GPL

;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program; if not, write to the Free Software
;; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

;;}}}

;;; Commentary:

;; This package add some more user control functions to EMMS, including
;; pause, seek, repeat. When "repeat", an or two extra stars will be
;; inserted in front of the `emms-mode-line-string', stands for
;; repeating current track and all tracks respectively.

;; Note: pause and seek only works with mplayer's slave mode. Put
;; something like this to your .emacs first:
;;
;; (setq emms-player-mplayer-command-name "mplayer"
;;       emms-player-mplayer-parameters '("-slave")
;;       emms-player-list
;;       '(emms-player-mplayer
;; 	   emms-player-mplayer-playlist
;; 	   emms-player-mpg321
;; 	   emms-player-ogg123))

;; To use, put this file to your load-path and the following(together
;; with your other emms's settings) to your .emacs:
;; 
;;     (require 'emms-player-extensions)

;;; Change Log

;; v 0.21 [2005/07/18 16:42:51] In function `emms-seek', add a call for
;;        `emms-lyric-seek' (defined in `emms-lyric.el').

;; v 0.2 [2005/07/17 20:22:53] And `emms-player-paused-p',
;;       `emms-player-player-paused-hook', and some modifications to
;;       cooperate with another package(`emms-lyric.el'). Rename this
;;       file from emms-patch.el to emms-player-extensions.el.

;; v 0.1 The initial version. Add `emms-pause', `emms-seek',
;;       `emms-repeat-curr', `emms-unrepeat-curr', `emms-repeat-all',
;;       `emms-unrepeat-all'.

;;; Todo:

;; Hopefully, this package will change synchronically with
;; `emms-lyric.el' for cooperations.

;;; Codes:

;; Version control
(defvar emms-player-extensions-version "0.2 $Revision: 1.1 $"
  "EMMS player extensions version string.")
;; $Id: emms-player-extensions.el,v 1.1 2005/07/20 15:49:52 xwl Exp $

(require 'emms)

;;; Variables:

(defvar emms-player-paused-p nil
  "The EMMS player paused.")

(defvar emms-player-paused-hook nil
  "*Hook run when an EMMS player pauses playing.")

;; (defvar emms-player-seeked-hook nil
;;   "*Hook run when an EMMS player seeks forward or backward.")

;;; User Interfaces:

(defun emms-pause ()
  "Pause the current player."
  (interactive)
  (when emms-player-playing-p
    (if (equal emms-player-playing-p
	       'emms-player-mplayer)
	(funcall (emms-player-get emms-player-playing-p 'pause))
      (message "Pause is only available for mplayer."))))

(defun emms-seek (second)
  "Seek forward/backward.

SECOND is a string like \"-10\"."
  (interactive)
  (when emms-player-playing-p
    (if (equal emms-player-playing-p
	       'emms-player-mplayer)
	(funcall (emms-player-get emms-player-playing-p 'seek) second)
      (message "Seek is only available for mplayer."))))

(defun emms-repeat-curr ()
  "Repeat current track."
  (interactive)
  (when emms-player-playing-p
    (funcall (emms-player-get emms-player-playing-p 'repeat-curr))))

(defun emms-unrepeat-curr ()
  "Stop repeating current track."
  (interactive)
  (when emms-player-playing-p
    (funcall (emms-player-get emms-player-playing-p 'unrepeat-curr))))

(defun emms-repeat-all ()
  "Repeat all tracks."
  (interactive)
  (when emms-player-playing-p
    (funcall (emms-player-get emms-player-playing-p 'repeat-all))))

(defun emms-unrepeat-all ()
  "Stop repeating all tracks."
  (interactive)
  (when emms-player-playing-p
    (funcall (emms-player-get emms-player-playing-p 'unrepeat-all))))

;;; Functions:

;; pause, seek(mplayer only)
(emms-player-set emms-player-mplayer
		 'pause
		 'emms-player-mplayer-pause)

(emms-player-set emms-player-mplayer
		 'seek
		 'emms-player-mplayer-seek)

(defun emms-player-mplayer-pause ()
  "Depends on mplayer's -slave mode."
  (process-send-string
   emms-player-simple-process-name "pause\n")
  (if emms-player-paused-p
      (setq emms-player-paused-p nil)
    (setq emms-player-paused-p t))
  (run-hooks 'emms-player-paused-hook))

(defun emms-player-mplayer-seek (sec)
  "Depends on mplayer's -slave mode."
  (process-send-string
   emms-player-simple-process-name
   (concat "seek " sec "\n"))
  (emms-lyric-seek sec))

;; repeat(any player)
(mapcar '(lambda (player)		; set up
	   (mapcar* '(lambda (name value)
		       (emms-player-set player
					name
					value))
		    '(repeat-curr
		      unrepeat-curr
		      repeat-all
		      unrepeat-all)
		    '(emms-player-any-repeat-curr
		      emms-player-any-unrepeat-curr
		      emms-player-any-repeat-all
		      emms-player-any-unrepeat-curr)))
      emms-player-list)

(defun emms-player-any-repeat-curr ()
 (setq emms-repeat-curr-track t)
 (emms-set-repeat-curr-mark)
 (message "Repeat current track."))

(defun emms-player-any-unrepeat-curr ()
 (setq emms-repeat-curr-track nil)
 (emms-set-unrepeat-curr-mark)
 (message "Stop repeating current track."))

(defun emms-player-any-repeat-all ()
 (setq emms-repeat-curr-track nil
       emms-repeat-playlist t)
 (emms-set-repeat-all-mark)
 (message "Repeat all tracks."))

(defun emms-player-any-unrepeat-all ()
 (setq emms-repeat-curr-track nil
       emms-repeat-playlist nil)
 (emms-set-unrepeat-all-mark)
 (message "Stop Repeating all tracks."))


(defun emms-set-repeat-curr-mark ()
  "Add a star at front indicating repeats."
  (setq emms-mode-line-format " [ * %s ] "
	emms-mode-line-string
	(funcall emms-mode-line-mode-line-function)))

(defun emms-set-unrepeat-curr-mark ()
  "Remove star mark at front."
  (setq emms-mode-line-format " [ %s ] "
	emms-mode-line-string
	(funcall emms-mode-line-mode-line-function)))

(defun emms-set-repeat-all-mark ()
  "Add a star at front indicating repeats."
  (setq emms-mode-line-format " [ ** %s ] "
	emms-mode-line-string
	(funcall emms-mode-line-mode-line-function)))

(defun emms-set-unrepeat-all-mark ()
  "Remove star mark at front."
  (setq emms-mode-line-format " [ %s ] "
	emms-mode-line-string
	(funcall emms-mode-line-mode-line-function)))


(provide 'emms-player-extensions)

;;; emms-player-extensions.el ends here
